import subprocess


def connect(device_id):
    subprocess.call("adb connect %s" % (device_id), shell=True,
                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)


def disconnect(device_id):
    subprocess.call("adb disconnect %s" % (device_id), shell=True,
                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)



from optparse import OptionParser
from parallel import ParallelType
import locale.en as locale
import sys

def parse_options():
    """
    Handle command-line options with optparse.OptionParser.

    Return list of arguments, largely for use in `parse_arguments`.
    """

    # Initialize
    parser = OptionParser(usage="galaxest [options]")

    parser.add_option(
        '-l','--devices',
        action='store_true',
        dest='want_list_devices',
        default=False,
        help=locale.HELP_DEVICES
    )

    parser.add_option(
        '-c','--connect',
        action="callback",
        callback=optional_argument,
        callback_args=(str, True),
        nargs=0,
        dest='want_connect',
        default=False,
        help=locale.HELP_CONNECT
    )

    parser.add_option(
        '-d','--disconnect',
        dest='device_id_to_disconnect',
        default=None,
        help=locale.HELP_DISCONNECT
    )

    parser.add_option(
        '--reset-config',
        action='store_true',
        dest='want_reset_config',
        default=False,
        help=locale.HELP_RESET_CONFIG
    )

    parser.add_option(
        '-p','--show-config',
        action='store_true',
        dest='want_show_config',
        default=False,
        help=locale.HELP_SHOW_CONFIG
    )

    parser.add_option(
        '-r','--run',
        dest='test_suite',
        default=None,
        help=locale.HELP_RUN
    )

    parser.add_option(
        '-i','--my-devices',
        action='store_true',
        dest='want_my_devices',
        default=False,
        help=locale.HELP_MY_DEVICES
    )

    parser.add_option(
        '-o','--opts',
        dest='opts',
        default=None,
        help=locale.HELP_OPTS
    )

    parser.add_option(
        '--by-id',
        dest='parallel_id',
        default=None,
        help=locale.HELP_PARALLEL_ID
    )

    parser.add_option(
        '--by-n',
        dest='parallel_number',
        default=None,
        help=locale.HELP_PARALLEL_N
    )

    parser.add_option(
        '--by-os',
        dest='parallel_os',
        default=None,
        help=locale.HELP_PARALLEL_OS
    )

    parser.add_option(
        '--skip-disconnect',
        action='store_true',
        dest='skip_disconnect',
        default=False,
        help=locale.HELP_SKIP_DISCONNECT
    )

    # Finalize
    # Return three-tuple of parser + the output from parse_args (opt obj, args)
    opts, args = parser.parse_args()
    opts = _check_conflicted_opts(opts)
    return parser, opts, args

def optional_argument(option, opt_str, value, parser, typ, default_arg):
    try:
        next_arg = typ(parser.rargs[0])
    except Exception:
        next_arg = default_arg
    else:
        parser.rargs.pop(0)
    setattr(parser.values, option.dest, next_arg)

def _check_conflicted_opts(opts) :
    conflict = False

    if opts.want_connect != False and not opts.device_id_to_disconnect is None :
        print locale.ERROR_CD_TOGETHER
        conflict = True
    
    parallel_request = 0
    opts.parallel_type = None
    opts.parallel_specs = None
    if not opts.parallel_number is None :
        parallel_request += 1
        opts.parallel_type = ParallelType.AMOUNT
        opts.parallel_specs = opts.parallel_number
    if not opts.parallel_os is None :
        parallel_request += 1
        opts.parallel_type = ParallelType.OS
        opts.parallel_specs = opts.parallel_os
    if not opts.parallel_id is None :
        parallel_request += 1
        opts.parallel_type = ParallelType.DEVICE_ID
        opts.parallel_specs = opts.parallel_id
    
    if parallel_request > 1 :
        print locale.ERROR_CHOOSE_PARALLEL_TYPE
        conflict = True
    
    if conflict :
        sys.exit(0)

    return opts


import inquirer
import labs
import automations
import devices.stf as stf
import config
import locale.en as locale


def init():
    setup_status = True
    print locale.INFO_SETUP_FIRST
    print locale.INFO_SETUP_SUB
    reset_q = [
        inquirer.List("ready",
                      message=locale.INFO_PROMPT_READY,
                      choices=[locale.INFO_YES, locale.INFO_NO]
                      )
    ]
    reset_answers = inquirer.prompt(reset_q)
    if reset_answers["ready"] == locale.INFO_NO:
        config.data.reset = False
        return False
    
    if setup_status:
        device_sources_q_key = "device_sources"
        device_sources_q = [
            inquirer.Checkbox(device_sources_q_key,
                              message=locale.INFO_SETUP_Q_DEVICE_FARM,
                              choices=[labs.KEY_DEVICELAB_OPENSTF]
                              )
        ]
        device_sources_answers = inquirer.prompt(device_sources_q)
        config.data.device_sources = device_sources_answers[device_sources_q_key]
        setup_status = setup_device_sources()

    if setup_status:
        framework_q_key = "framework"
        framework_q = [
            inquirer.List(framework_q_key,
                        message=locale.INFO_SETUP_Q_TEST_FW,
                        choices=[automations.KEY_FRAMEWORK_KATALON]
                        )
        ]
        framework_answers = inquirer.prompt(framework_q)
        config.data.framework = framework_answers[framework_q_key]
        setup_status = setup_automation()

    if setup_status:
        config.data.reset = False
        print locale.SUCCESS_SETUP

    return setup_status

def setup_automation():
    setup_status = True
    if config.data.framework == automations.KEY_FRAMEWORK_KATALON :
        setup_status = setup_katalon()
    return setup_status

def setup_katalon():
    setup_status = True
    init_katalon_q = [
        inquirer.Text('katalon_path',
                      message=locale.INFO_SETUP_Q_KATALON_PATH,
                      default=config.data.katalon_app if "katalon_app" in config.data else None
                      ),
        inquirer.Text('project_file',
                      message=locale.INFO_SETUP_Q_KATALON_PROJECT_FILE,
                      default=config.data.katalon_project_file if "katalon_project_file" in config.data else None
                      )
    ]
    init_katalon_answers = inquirer.prompt(init_katalon_q)
    config.data.katalon_app = init_katalon_answers["katalon_path"].strip() if not init_katalon_answers["katalon_path"].endswith(".app") else '%s/Contents/MacOS/katalon' % (init_katalon_answers["katalon_path"])
    config.data.katalon_project_file = init_katalon_answers["project_file"].strip()
    config.data.katalon_project_path = config.data.katalon_project_file.rsplit('/', 1)[0]
    config.data.katalon = True
    return setup_status

def setup_device_sources():
    setup_status = True
    for source in config.data.device_sources:
        if source == labs.KEY_DEVICELAB_OPENSTF:
            setup_status = setup_stf()
    return setup_status

def setup_stf():
    setup_status = True
    init_stf_q = [
        inquirer.Text('host',
                      message=locale.INFO_SETUP_Q_STF_HOST,
                      default=config.data.stf_host if "stf_host" in config.data else None
                      ),
        inquirer.Text('token',
                      message=locale.INFO_SETUP_Q_STF_TOKEN,
                      default=config.data.stf_token if "stf_token" in config.data else None
                      ),
    ]
    init_stf_answers = inquirer.prompt(init_stf_q)
    config.data.stf_host = init_stf_answers["host"].strip()
    config.data.stf_token = init_stf_answers["token"].strip()
    config.data.stf = True
    stf.init_stf()
    print locale.INFO_SETUP_STF_TEST_CONN
    if not stf.stf_api.healthy():
        print locale.ERROR_STF_TEST_CONN
        setup_status = False
    print ""
    return setup_status



import json, subprocess
from tweak import Config

data = Config(name="galaxest")

def show_config():
    configuration = data
    proc = subprocess.Popen("cat %s/config.json" % (configuration.user_config_dir), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    print json.dumps(json.loads(str(proc.stdout.read())), indent=4, sort_keys=True)



#HELP
HELP_DEVICES = "List available devices in your galaxest"
HELP_CONNECT = "Connect to a device in galaxest to your machine"
HELP_DISCONNECT = "Disconnect a galaxest's device from your machine"
HELP_RESET_CONFIG = "Reset your spaceship configurations"
HELP_SHOW_CONFIG = "Show your spaceship configurations"
HELP_RUN = "Run a test from your automation framework"
HELP_MY_DEVICES = "See devices from galaxest connected to this machine."
HELP_OPTS = "Specify more arguments for your test run"
HELP_PARALLEL_N = "Run the test by specifying desired amount of devices"
HELP_PARALLEL_OS = "Run the test by specifying desired os versions"
HELP_PARALLEL_ID = "Run the test by specifying device ids"
HELP_SKIP_DISCONNECT = "Skip auto disconnect device after test finished"

#SUCCESS
SUCCESS_SETUP = "Done! Your spaceship is ready to explore the galaxest"
SUCCESS_DEVICE_CONNECT = "successfully connected to your adb as"
SUCCESS_DEVICE_DISCONNECT = "device successfully disconnected from your machine"

#ERROR
ERROR_CD_TOGETHER = "Can't connect and disconnect at one time. Please choose one."
ERROR_DISCONNECT_NO_ID = "Can't find device to be disconnected. Please specify the device's serial!"
ERROR_DISCONNECT_ID_NOT_FOUND = "galaxest doesn't see this device connected"
ERROR_CONNECT_DEVICE = "Can't connect to the desired device. Wanna try another device?"
ERROR_DEVICE_BUSY = "Device is currently running another automation. Please re-run after it finish or choose another device."
ERROR_DEVICE_UNAVAILABLE = "The desired device is currently unavailable. You can see available devices by run 'galaxest -l'"
ERROR_DEVICE_NOT_FOUND = "Can't find any devices match to your criteria to run the test"
ERROR_STF_TEST_CONN = "Failed to connect to your OpenSTF!"
ERROR_TESTFW_NOT_FOUND = "Failed to find your Automation framework"
ERROR_CONNECT_ADB_KEY = "Somethings wrong with your account's adb key. Please recheck it"
ERROR_ADB_KEY_NOT_FOUND = "You don't have any adb key. Please check it on Settings page at your OpenSTF Dashboard."
ERROR_DISCONNECT_HUNG = "It may failed to disconnect the device"
ERROR_STF_CONFIG = "Something wrong with the OpenSTF configuration. Please check your app_config.json first."
ERROR_STF_FETCH_DEVICES = "failed to get list of devices"
ERROR_STF_STOP_DEVICE = "failed to stop use the device"
ERROR_STF_USE_DEVICE = "failed to use the device"
ERROR_CHOOSE_PARALLEL_TYPE = "Please choose only one type of parallel execution"

#INFO
INFO_SETUP_FIRST = "Hi! This is your first time exploring galaxest."
INFO_SETUP_SUB = "Let's setup your spaceship first!"
INFO_SETUP_Q_TEST_FW = "What UI automation framework do you use?"
INFO_SETUP_Q_DEVICE_FARM = "What are device farm sources do you have? (press right arrow button on your keyboard to choose)"
INFO_SETUP_Q_KATALON_PATH = "Where is your Katalon Application?"
INFO_SETUP_Q_KATALON_PROJECT_PATH = "Where is your Katalon project?"
INFO_SETUP_Q_KATALON_PROJECT_FILE = "Where is your katalon .prj file?"
INFO_SETUP_Q_STF_HOST = "What's your OpenSTF host (e.g. http://172.0.0.1:7100)"
INFO_SETUP_Q_STF_TOKEN = "What's your OpenSTF user API Token"
INFO_SETUP_STF_TEST_CONN = "Testing the connection to your OpenSTF..."
INFO_PROMPT_READY = "Are you ready?"
INFO_YES = "Yes"
INFO_NO = "No"
INFO_DEVICE_CONNECTING = "connecting the device..."
INFO_DEVICE_DISCONNECTING = "disconnecting the device..."
INFO_LOGGED_IN = "You are logged in as"
INFO_KATALON_START = "Starting Katalon automation at"



import requests
import galaxest.locale.en as locale

class Stf(object):

    def __init__(self, host, token):
        self.host = host if host[len(host)-1] == '/' else '%s/' % (host)
        self.token = token
        self.user = None
        self.adb_key_exist = False
        self.auth = "Bearer %s" % (self.token)

    def healthy(self):
        if not self.host is None and not self.token is None:
            self.user = self.get_user()
            if not self.user is None:
                print "%s %s (%s)" % (
                    locale.INFO_LOGGED_IN, self.user["name"], self.user["email"])
                if len(self.user["adbKeys"]) > 0:
                    self.adb_key_exist = True
                    return True
                else:
                    print locale.ERROR_ADB_KEY_NOT_FOUND
        else:
            print locale.ERROR_STF_CONFIG
        return False

    def get_user(self):
        headers = {
            "Authorization": self.auth
        }
        path = "api/v1/user"
        res = requests.get(self.host+path, headers=headers).json()
        if "success" in res and res["success"] and "user" in res:
            return res["user"]
        else:
            return None

    def get_devices(self):
        headers = {
            "Authorization": self.auth
        }
        path = "api/v1/devices"
        res = requests.get(self.host+path, headers=headers).json()
        available_devices = list()
        if "success" in res and res["success"] and "devices" in res:
            devices = res["devices"]
            for device in devices:
                if device["ready"] and device["present"] and device["owner"] is None:
                    available_devices.append(device)
        else:
            print "failed to get list of devices"
        return available_devices

    def get_device(self, serial):
        headers = {
            "Authorization": self.auth
        }
        path = "api/v1/devices/%s" % (serial)
        return requests.get(self.host+path, headers=headers).json()

    def get_user_devices(self):
        headers = {
            "Authorization": self.auth
        }
        path = "api/v1/user/devices"
        res = requests.get(self.host+path, headers=headers).json()
        user_devices = list()
        if "success" in res and res["success"] and "devices" in res:
            user_devices = res["devices"]
        else:
            print locale.ERROR_STF_FETCH_DEVICES
        return user_devices

    def user_use_device(self, serial):
        headers = {
            "Authorization": self.auth
        }
        body = {
            "serial": serial
        }
        path = "api/v1/user/devices"
        res = requests.post(self.host+path, headers=headers, json=body).json()
        if "success" in res and res["success"]:
            return True
        else:
            print res["description"] if "description" in res else locale.ERROR_STF_USE_DEVICE
            return False

    def user_stop_use_device(self, serial):
        headers = {
            "Authorization": self.auth
        }
        path = "api/v1/user/devices/%s" % serial
        res = requests.delete(self.host+path, headers=headers).json()
        if "success" in res and res["success"]:
            return True
        else:
            print locale.ERROR_STF_STOP_DEVICE
            return False

    def device_remote_connect(self, serial):
        headers = {
            "Authorization": self.auth
        }
        path = "api/v1/user/devices/%s/remoteConnect" % (serial)
        res = requests.post(self.host+path, headers=headers).json()
        return res["remoteConnectUrl"] if "remoteConnectUrl" in res else None

    def device_remote_disconnect(self, serial):
        headers = {
            "Authorization": self.auth
        }
        path = "api/v1/user/devices/%s/remoteConnect" % (serial)
        res = requests.delete(self.host+path, headers=headers).json()
        return True if "success" in res and res["success"] else False



import sys
import json
import device
import automation
import setup
import config
import time
import locale.en as locale
import parallel

options = None

def run():
    # --show-config
    if options.want_show_config:
        config.show_config()
        sys.exit(0)

    # --reset-config
    if options.want_reset_config:
        config.data.reset = True

    # check if configuration had configured
    config_blank = True
    if config.data and "reset" in config.data and config.data.reset is False:
        config_blank = False
    if config_blank:
        setup.init()
    
    if not "devices_connected" in config.data :
        config.data.devices_connected = {}

    # --devices
    if options.want_list_devices:
        device.print_available_devices()
        sys.exit(0)

    if options.want_my_devices:
        device.print_my_devices()
        sys.exit(0)

    # --connect
    if options.want_connect != False :
        if options.want_connect == True :
            any_spec = parallel.ParallelExecution(parallel.ParallelType.AMOUNT, 1)
            devices_selected = device.auto_choose_device(any_spec, False)
        else :
            desired_device = device.init(options.want_connect)
            device.connect(desired_device)

    # --disconnect
    if not options.device_id_to_disconnect is None :
        if options.device_id_to_disconnect in config.data.devices_connected :
            device.disconnect(options.device_id_to_disconnect)
        else :
            print locale.ERROR_DISCONNECT_ID_NOT_FOUND
    
    # --run
    if not options.test_suite is None :
        devices_selected = list()
        devices_selected = device.find_devices_to_run(options.parallel_type, options.parallel_specs)
        if len(devices_selected) > 0 :
            automation.run(options.test_suite, devices_selected, options.opts)
            if not options.skip_disconnect :
                automation.wait_until_finish(devices_selected)
                for ds in devices_selected :
                    device.disconnect(ds.serial)
        else :
            print locale.ERROR_DEVICE_NOT_FOUND






import subprocess
import os
import datetime
import galaxest.locale.en as locale

class App (object) :

    def __init__(self, app_path, project_path, project_file):
        self.app_name = "/katalon"
        self.app_path = app_path[:-len(self.app_name)] if app_path.endswith(self.app_name) else self.app_name
        self.project_path = project_path
        self.project_file = project_file

class Runner (object) :

    def __init__(self, katalon):
        self.katalon = katalon
        self.test_suite = None
        self.device = None
        self.run_options = None
        self.run_background = None
        self.report_path = None
        self.timestamp = None

    def run(self, test_suite, device, opts, timestamp, background):
        self.test_suite = test_suite
        self.device = device
        self.run_options = opts
        self.timestamp = timestamp
        self.run_background = background
        self.run_options = opts if not opts is None else ""
        self.report_path = self.create_report_path()
        execution_file = self.generate_test_run_files()
        print "%s %s" % (locale.INFO_KATALON_START, device.name)
        subprocess.call('"%s"' % (execution_file), shell=True)

    def generate_test_run_files(self):
        log_file = self.create_run_log_file()
        runner_file = self.create_katalon_runner(log_file)
        screen_file = self.create_screen_runner(runner_file)
        return screen_file

    def create_katalon_runner(self, log_file):
        command = './katalon --args -noSplash -runMode=console -consoleLog -testSuitePath="Test Suites/%s" -deviceId="%s" -browserType="Android" %s %s' % (
            self.test_suite, self.device.adb_id, self.get_report_folder(), self.run_options)
        if not '-projectPath=' in self.run_options :
            command = "%s -projectPath=%s" % (command, self.katalon.project_file)
        command = '%s 2>&1 | tee "%s"' % (command, log_file)
        filename = "%s/katalon_runner" % (self.report_path)
        f = open(filename, 'w+')
        f.write(
            """
        #!/bin/bash
        cd %s
        %s
        """ % (self.katalon.app_path, command)
        )
        subprocess.call('chmod +x "%s"' % (filename), shell=True)
        return filename

    def create_screen_runner(self, runner_file):
        filename = "%s/screen_runner" % (self.report_path)
        command = 'screen -dmS katalon "%s"' % (runner_file) if self.run_background else '"%s"' % (runner_file)
        f = open(filename, 'w+')

        f.write(
            """
        #!/bin/bash
        %s
        """ % (command)
        )
        subprocess.call('chmod +x "%s"' % (filename), shell=True)
        return filename

    def create_run_log_file(self):
        filename = "%s/run.log" % (self.report_path)
        subprocess.call('touch "%s"' % (filename), shell=True)
        return filename

    def get_report_folder(self):
        if not self.run_options is None and not "-reportFolder=" in self.run_options:
            return '-reportFolder="%s"' % (self.report_path)
        return ''

    def create_report_path(self):
        time_format = '%H.%M.%S %d-%m-%Y'
        report_path = "%s/Reports/%s/%s/%s_%s" % (self.katalon.project_path, self.test_suite, self.timestamp.strftime(time_format), self.device.name, self.device.serial)
        os.makedirs(report_path, 0755)
        return report_path


import time
import labs
import config
import automation
import locale.en as locale
import parallel
from parallel import ParallelType

import devices.stf

sources = config.data.device_sources if "device_sources" in config.data else None

def init(device_id):
    for source in sources:
        if source == labs.KEY_DEVICELAB_OPENSTF:
            device = devices.stf.Stf(device_id)
            if device.is_exist():
                return device
    return None

def print_available_devices():
    print "[OpenSTF]"
    print "%-20s\t%-10s\t%s" % ("[Device Name]", "[Version]", "[Device ID]")
    devices.stf.print_devices()

def print_my_devices():
    _check_connected_devices()
    devices = config.data.devices_connected
    print "%-10s\t%-20s\t%-10s\t%-20s\t%s" % (
        "[Source]", "[Device Name]", "[Version]", "[Device ID]", "[ADB ID]",)
    for key in devices:
        device = devices[key]
        source = device.source if "source" in device else None
        name = device.name if "name" in device else None
        version = device.version if "version" in device else None
        serial = device.serial if "serial" in device else None
        adb_id = device.adb_id if "adb_id" in device else None
        print "%-10s\t%-20s\t%-10s\t%-20s\t%s" % (source,name,version,serial,adb_id)

def connect(device):
    if device.is_available():
        connected_device = _create_connection(device)
        if not connected_device is None:
            return connected_device
    else:
        print locale.ERROR_DEVICE_UNAVAILABLE
    return None

def disconnect(device_id):
    device = init(device_id)
    device.remote_url = config.data.devices_connected[device_id]["adb_id"]
    device.disconnect()
    _check_connected_devices()

def get_available_devices():
    available_devices = []
    for source in sources:
        if source == labs.KEY_DEVICELAB_OPENSTF:
            available_devices = available_devices + devices.stf.get_devices()
    return available_devices

def find_devices_to_run(parallel_type, parallel_specs):
    devices_selected = list()
    if not parallel_type is None :
        parallel_execution = parallel.ParallelExecution(parallel_type, parallel_specs)
        devices_selected = auto_choose_device(parallel_execution)
    else :
        any_spec = parallel.ParallelExecution(parallel.ParallelType.AMOUNT, 1)
        devices_selected = auto_choose_device(any_spec)
    return devices_selected

def auto_choose_device(parallel_data, use_connected_devices=True):
    devices_selected = []
    if use_connected_devices :
        devices_selected = _use_connected_devices(parallel_data, devices_selected)
    devices_selected = _use_available_devices(parallel_data, devices_selected)
    return devices_selected

def _check_connected_devices():
    for source in sources:
        list_disconnected = []
        if source == labs.KEY_DEVICELAB_OPENSTF:
            list_disconnected = list_disconnected + devices.stf.get_disconnected_devices()
    for serial in list_disconnected:
        del config.data.devices_connected[serial]

def _is_connected_device_usable(device_connected):
    if not automation.is_using_device(device_connected.adb_id):
        return True
    return False

def _is_device_spec_match(parallel_data, desired_device):
    spec_match = True
    spec = None
    if parallel_data.type_val != ParallelType.AMOUNT :
        spec_match = False
        for i_spec in parallel_data.specs :
            if _is_spec_match(parallel_data.type_val, i_spec, desired_device) :
                spec = i_spec
                spec_match = True
    return spec_match, spec

def _is_spec_match(parallel_type, spec, desired_device):
    if parallel_type == ParallelType.OS and desired_device.version.startswith(spec) :
        return True
    elif parallel_type == ParallelType.DEVICE_ID and desired_device.serial == spec :
        return True
    return False
        

def _use_connected_devices(parallel_data, devices_selected=[]):
    _check_connected_devices()
    if len(config.data.devices_connected) > 0:
        for key in config.data.devices_connected:
            if len(devices_selected) < parallel_data.specs_amount:
                device_connected = config.data.devices_connected[key]
                spec_match, spec = _is_device_spec_match(parallel_data, device_connected) 
                if spec_match and _is_connected_device_usable(device_connected):
                    parallel_data.remove_spec(spec)
                    devices_selected.append(device_connected)
            else : break
    return devices_selected

def _use_available_devices(parallel_data, devices_selected=[]):
    if len(devices_selected) < parallel_data.specs_amount:
        available_devices = get_available_devices()
        n_more_devices = parallel_data.specs_amount - len(devices_selected)
        if len(available_devices) >= n_more_devices:
            counter_device = 0
            for x in range(0, n_more_devices):
                device_found = False
                while not device_found and counter_device < len(available_devices):
                    serial = available_devices[counter_device]["serial"]
                    desired_device = init(serial)
                    spec_match, spec = _is_device_spec_match(parallel_data, desired_device)
                    if spec_match and desired_device.is_available():
                        connected_device = connect(desired_device)
                        if not connected_device is None:
                            parallel_data.remove_spec(spec)
                            devices_selected.append(
                                config.data.devices_connected[serial])
                            device_found = True
                    counter_device += 1
    return devices_selected

def _register_connected_device(device):
    if not "devices_connected" in config.data:
        config.data.devices_connected = dict()
    config.data.devices_connected[device.serial] = {
        "serial": device.serial,
        "adb_id": device.remote_url,
        "version": device.version,
        "name": device.name,
        "source": device.source
    }
    config.data.save()
    return config.data.devices_connected[device.serial]


def _create_connection(device):
    connection_status = device.connect()
    if connection_status:
        just_registered_device = _register_connected_device(device)
        return just_registered_device
    return None



import automations
import automations.katalon as katalon
import locale.en as locale
import config
import subprocess
import time
import datetime

framework = config.data.framework if "framework" in config.data else None


def run(test_suite, devices_selected, opts=None, background=False):
    if len(devices_selected) > 1:
        background = True
    now = datetime.datetime.now()
    if framework == automations.KEY_FRAMEWORK_KATALON:
        katalon_app = katalon.App(config.data.katalon_app, config.data.katalon_project_path, config.data.katalon_project_file)
        for device in devices_selected:
            runner = katalon.Runner(katalon_app)
            runner.run(test_suite, device, opts, now, background)
    else:
        print locale.ERROR_TESTFW_NOT_FOUND


def is_using_device(local_id):
    if framework == automations.KEY_FRAMEWORK_KATALON:
        identifier = "deviceId=%s" % (local_id)
    proc = subprocess.Popen("ps xu | grep '%s' | grep -v grep | awk '{ print $2 }'" % (
        identifier), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output = proc.stdout.read()
    return True if output != "" else False

def _populate_adb_ids(devices_selected):
    adb_ids = list()
    for ds in devices_selected :
        adb_ids.append(ds.adb_id)
    return adb_ids

def wait_until_finish(devices_selected):
    automation_finish = False
    time.sleep(10)
    while not automation_finish:
        automation_finish = True
        for ds in devices_selected:
            if not 'being_used' in ds :
                ds.being_used = True
            if ds.being_used :
                ds.being_used = is_using_device(ds.adb_id)
                automation_finish = automation_finish and not ds.being_used
        if not automation_finish :
            time.sleep(30)


# encoding: utf-8

from setuptools import setup, find_packages, Command
import sys
import os
import re
import ast

# read the contents of your README file
this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(
    name='galaxest',
    version='0.0.8',
    description="Run test from anywhere to anywhere in the galaxy",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Fernanda Panca Prima',
    author_email='pancaprima8@gmail.com',
    url='https://github.com/pancaprima/galaxest',
    license='MIT',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=["requests","tweak", "inquirer", "enum"],
    entry_points={
        'console_scripts': [
            'galaxest = galaxest.main:main',
        ]
    },
)



import sys
import galaxest
import runner
import args_parser

def main():
    parser, options, arguments = args_parser.parse_options()
    runner.options = options
    runner.run()
    sys.exit(0)

if __name__ == '__main__':
    main()



import galaxest.config as config
import galaxest.adb as adb
import galaxest.labs as labs
import galaxest.locale.en as locale
from galaxest.labs.stf import Stf as StfApi

stf_api = None


def init_stf():
    global stf_api
    stf_api = StfApi(config.data.stf_host, config.data.stf_token)


if "stf" in config.data and config.data.stf:
    init_stf()

def get_devices():
    return stf_api.get_devices()

def get_disconnected_devices():
    list_disconnected = []
    user_devices = stf_api.get_user_devices()
    for key in config.data.devices_connected :
        still_connected = False
        device_connected = config.data.devices_connected[key]
        for user_device in user_devices : 
            if user_device["serial"] == key or device_connected.source != labs.KEY_DEVICELAB_OPENSTF :
                still_connected = True
        if not still_connected :
            list_disconnected.append(device_connected.serial)
    return list_disconnected

def print_devices():
    available_devices = get_devices()
    if len(available_devices) > 0:
        for device in available_devices:
            device_name = "%s(%s)" % (
                device["manufacturer"].strip(), device["model"].strip())
            print "%-20s\t%-10s\t%s" % (device_name, device['version'], device["serial"])  


class Stf(object):
    def __init__(self, serial):
        self.source = labs.KEY_DEVICELAB_OPENSTF
        self.serial = serial
        self.remote_url = None
        res = stf_api.get_device(self.serial)
        self.info = res["device"] if "device" in res else None
        self.exist = res["success"] if "success" in res else False
        if not self.info is None :
            self.name = "%s(%s)" % (self.info["manufacturer"].strip(), self.info["model"].strip())
            self.version = self.info["version"]
        if serial in config.data.devices_connected :
            self.remote_url = config.data.devices_connected[serial].adb_id

    def connect(self):
        print locale.INFO_DEVICE_CONNECTING
        if stf_api.user_use_device(self.serial):
            self.remote_url = stf_api.device_remote_connect(self.serial)
            if not self.remote_url is None:
                adb.connect(self.remote_url)
                print "%s %s %s %s" % (
                    self.serial,
                    self.name,
                    locale.SUCCESS_DEVICE_CONNECT,
                    self.remote_url)
                return True
            else:
                print locale.ERROR_CONNECT_ADB_KEY
        else:
            print locale.ERROR_CONNECT_DEVICE
        return False

    def disconnect(self):
        print locale.INFO_DEVICE_DISCONNECTING
        adb.disconnect(self.remote_url)
        if stf_api.device_remote_disconnect(self.serial) and stf_api.user_stop_use_device(self.serial):
            adb.disconnect(self.remote_url)
            print locale.SUCCESS_DEVICE_DISCONNECT
            return True
        else:
            print locale.ERROR_DISCONNECT_HUNG
        return False

    def is_available(self):
        if self.exist and self.info["owner"] is None:
            return True
        return False

    def is_exist(self):
        return self.exist



import enum

class ParallelType(enum.Enum): 
    AMOUNT = 0
    OS = 1
    DEVICE_ID = 2

class ParallelExecution(object):

    def __init__(self, new_type, new_specs):
        self.type_val = None
        self.specs = None
        self.specs_amount = 0
        self.set_parallel_type(new_type)
        self.parse_specs(new_specs)

    def set_parallel_type(self, val):
        self.type_val = val
    
    def remove_spec(self, spec):
        if spec in self.specs:
            self.specs.remove(spec)

    def parse_specs(self, val):
        self.specs = []
        self.specs_amount = 1
        if self.type_val == ParallelType.AMOUNT :
            self.specs_amount = int(val)
        else :
            self.specs = val.split(",")
            self.specs_amount = int(len(self.specs))

